// Copyright 2024 Bytedance Inc. All Rights Reserved.
// Author: tianlei.richard@qq.com (tianlei.richard)

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include "spdlog/spdlog.h"

#include "camera.h"
#include "common.h"
#include "minus_renderer.h"

int main(int argc, char *argv[]) {
  spdlog::set_level(spdlog::level::trace);

  const std::string obj_path{argv[1]};

  const int resolution_width = 640;
  const int resolution_height = 320;
  const int far = -100;
  const int near = -1;
  const float fov = 120. / 180. * M_PI;
  const float aspect_ratio = static_cast<float>(resolution_width) /
                             static_cast<float>(resolution_height);

  cv::namedWindow("MinusRenderer", cv::WINDOW_AUTOSIZE);
  const auto start = std::chrono::steady_clock::now();
  Camera camera{Vector3d{0, 1, 0}};
  MinusRenderer renderer{near, far, fov, aspect_ratio};
  assert(renderer.load_mesh(obj_path));
  TransformMatrix translate{
      {1., 0., 0., 3.}, {0., 1., 0., 3.}, {0., 0., 1., -4.}, {0., 0., 0., 1.}};
  renderer.model_transform(translate);
  do {
    const auto elapse = std::chrono::duration_cast<std::chrono::milliseconds>(
                            std::chrono::steady_clock::now() - start)
                            .count();
    TransformMatrix move2origin{
        {1, 0, 0, 0}, {0, 1, 0, -3}, {0, 0, 1, 4}, {0, 0, 0, 1}};
    TransformMatrix rotation{
        Eigen::Transform<double, 3, Eigen::Affine>(
            Eigen::AngleAxisd(0.0625 * M_PI, Eigen::Vector3d::UnitX())
                .toRotationMatrix())
            .matrix()};
    TransformMatrix move_back{
        {1, 0, 0, 0}, {0, 1, 0, 3}, {0, 0, 1, -4}, {0, 0, 0, 1}};
    renderer.model_transform(move_back * rotation * move2origin);

    camera.set_position(Point3d{0, 0, 0});
    camera.set_gaze(Vector3d{0, 0, -1});
    renderer.view_transform(camera.get_view_transform());

    const auto &color_image =
        renderer.render(resolution_width, resolution_height);
    cv::imshow("MinusRenderer", color_image);
  } while (cv::waitKey(40) == -1);
  cv::destroyWindow("MinusRenderer");

  return 0;
}